#include "StdAfx.h"
#include "Sqlite3Statement.h"
#include <ftlNLS.h>

CSqlite3Statement::CSqlite3Statement(void)
{
	m_pSQLite = NULL;
	m_pstmt = NULL;
}

CSqlite3Statement::~CSqlite3Statement(void)
{
	Close();
}

int CSqlite3Statement::Close()
{
	if (m_pstmt)
	{
		sqlite3_finalize(m_pstmt);
		m_pstmt = NULL;
	}

	m_pSQLite = NULL;
	return 0;
}

int CSqlite3Statement::Create(sqlite3 *pSQLite,const char *szSQL)
{
	m_pSQLite = pSQLite;

	int nResult = sqlite3_prepare(pSQLite, szSQL, -1, &m_pstmt, NULL);
	if(nResult == SQLITE_OK)
	{
		sqlite3_reset(m_pstmt);
	}

	return nResult;
}


int CSqlite3Statement::Execute()
{
	return SQLITE_OK;
}

int CSqlite3Statement::BindTime(const char* szFieldName,CTime &tm)
{
	_ASSERT(m_pstmt);

	CString strTime = CTimeToCString(tm);
	return BindText(szFieldName,strTime);
}

int CSqlite3Statement::BindInt(CString fieldName,int val)
{
	_ASSERT(m_pstmt);
	FTL::CFConversion conv;
	return BindInt(conv.TCHAR_TO_UTF8(fieldName),val);
}

int CSqlite3Statement::BindInt(const char* szFieldName,int val)
{
	_ASSERT(m_pstmt);
	
	int nResult = SQLITE_ERROR;
	int nIndex = sqlite3_bind_parameter_index(m_pstmt,szFieldName);
	if (nIndex > -1)
	{
		nResult = sqlite3_bind_int(m_pstmt,nIndex,val);
	}

	return nResult;
}

int CSqlite3Statement::BindInt64(const char* szFieldName,__int64 val)
{
	_ASSERT(m_pstmt);
	int nResult = SQLITE_ERROR;
	int nIndex = sqlite3_bind_parameter_index(m_pstmt,szFieldName);
	if (nIndex > -1)
	{
		nResult = sqlite3_bind_int64(m_pstmt,nIndex,val);
	}
	
	return nResult;
}

int CSqlite3Statement::BindInt64(CString fieldName,__int64 val)
{
	_ASSERT(m_pstmt);

	FTL::CFConversion conv;
	return BindInt64(conv.TCHAR_TO_UTF8(fieldName),val);
}

int CSqlite3Statement::BindText(const char* szFieldName,const CString& val)
{
	FTL::CFConversion conv;
	return BindText(szFieldName, conv.TCHAR_TO_UTF8(val));
}

int CSqlite3Statement::BindText(const char* szFieldName,const char* val)
{
	_ASSERT(m_pstmt);

	int nResult = SQLITE_ERROR;
	
	int len = strlen(val);

	char *p = new char[len + 1];
	if (p)
	{
		strcpy(p,val);
		m_vecValue.push_back(p);
		int nIndex = sqlite3_bind_parameter_index(m_pstmt,szFieldName);
		if (nIndex > -1)
		{
			nResult = sqlite3_bind_text(m_pstmt,nIndex,p,len,SQLITE_STATIC);
		}
	}

	return nResult;
}

int CSqlite3Statement::BindBlob(const char* szFieldName,const unsigned char* val,int len)
{
	_ASSERT(m_pstmt);

	int nResult = -1;

	int nIndex = sqlite3_bind_parameter_index(m_pstmt,szFieldName);
	if (nIndex > -1)
	{
		nResult = sqlite3_bind_blob(m_pstmt,nIndex,val,len,SQLITE_STATIC);
	}

	return nResult;
}

int CSqlite3Statement::Next()
{
	_ASSERT(m_pstmt);
	int nRet = sqlite3_step(m_pstmt);

	for (int i = 0; i < m_vecValue.size(); i++)
	{
		delete[] m_vecValue[i];
	}

	m_vecValue.clear();
	return nRet;
}

int CSqlite3Statement::GetFieldIndex(const char *szFieldName)
{
	int nIndex = -1;
	int count = sqlite3_column_count(m_pstmt);
	for (int i = 0; i < count; i++)
	{
		const char *f = (const char*)sqlite3_column_name(m_pstmt,i);
		if (strcmpi(szFieldName,f) == 0)
		{
			nIndex = i;
			break;
		}
	}

	return nIndex;
}

int CSqlite3Statement::GetFieldIndex(CString fieldName)
{
	FTL::CFConversion conv;
	return GetFieldIndex(conv.TCHAR_TO_UTF8(fieldName));
}

int CSqlite3Statement::GetIntValue(CString fieldName)
{
	int val = 0;
	int nIndex = GetFieldIndex(fieldName);
	if (nIndex >= 0)
	{
		val = sqlite3_column_int(m_pstmt,nIndex);
	}
	
	return val;
}

int CSqlite3Statement::GetIntValue(const char *szFieldName)
{
	int val = 0;
	int nIndex = GetFieldIndex(szFieldName);
	if (nIndex >= 0)
	{
		val = sqlite3_column_int(m_pstmt,nIndex);
	}

	return val;
}

int CSqlite3Statement::GetBlobLength(CString fieldName)
{
	int len = 0;
	int nIndex = GetFieldIndex(fieldName);
	if (nIndex >= 0)
	{
		len = sqlite3_column_bytes(m_pstmt,nIndex);
	}

	return len;
}

unsigned char* CSqlite3Statement::GetBlobValue(CString fieldName)
{
	unsigned char *pdata = NULL;
	int nIndex = GetFieldIndex(fieldName);
	if (nIndex >= 0)
	{
		pdata = (unsigned char*)sqlite3_column_blob(m_pstmt,nIndex);
	}

	return pdata;
}

unsigned char* CSqlite3Statement::GetBlobValue(const char * szFieldName)
{
	unsigned char *pdata = NULL;
	int nIndex = GetFieldIndex(szFieldName);
	if (nIndex >= 0)
	{
		pdata = (unsigned char*)sqlite3_column_blob(m_pstmt,nIndex);
	}

	return pdata;
}

CString CSqlite3Statement::GetTextValue(const char *szFieldName)
{
	int nIndex = GetFieldIndex(szFieldName);
	if (nIndex >= 0)
	{
		const char *p = (const char*)sqlite3_column_text(m_pstmt,nIndex);
		if (p)
		{
			return UTF8ToUnicode(p,strlen(p));
		}
	}

	return _T("");
}


CTime CSqlite3Statement::GetTime(const char *szFieldName)
{
	CString tm = GetTextValue(szFieldName);
	return StringToCTime(tm);
}
